const aws = require('aws-sdk');
const Task = require('data.task');
const nconf = require('../env');
const crypto = require('crypto');
const _ = require('lodash');

aws.config.update({
  accessKeyId: nconf.get('AWS_ACCESS_KEY_ID'),
  secretAccessKey: nconf.get('AWS_SECRET_ACCESS_KEY')
});

const s3 = new aws.S3();

let s3Handler = ({ req, res, path, sha, folder = 'www' }) => {
  let parts = _.compact(path.split('/'));
  let hasExt = /\..+$/.test(_.last(parts));
  let url = parts.concat(!hasExt ? 'index.html' : []).join('/');
  let params = {
    Bucket: nconf.get('AWS_BUCKET_NAME'),
    Key: `design-system/${sha}/${folder}/${url}`
  };

  let hash = crypto
    .createHash('md5')
    .update(params.Key)
    .digest('hex');
  let etag = req.headers['if-none-match'];
  // Instead of using the content hash as the ETag, we use the Key hash
  // because we know the files at a given Key will never change
  if (hash === etag) {
    return res.sendStatus(304);
  }
  s3.getObject(params, (err, data) => {
    if (err) return res.sendStatus(404);
    res.set('Content-Type', data.ContentType);
    res.set('Content-Length', data.ContentLength);
    res.set('ETag', hash);
    res.send(data.Body);
  });
};

const getSignedUrl = (sha, path) => {
  let params = {
    Bucket: nconf.get('AWS_BUCKET_NAME'),
    Key: `design-system/${sha}/${path}`,
    Expires: 20
  };

  return new Task((rej, res) =>
    s3.getSignedUrl(
      'getObject',
      params,
      (err, url) => (err ? rej(err) : res(url))
    )
  );
};

module.exports = { getSignedUrl, s3Handler };
