// Copyright (c) 2015-present, salesforce.com, inc. All rights reserved
// Licensed under BSD 3-Clause - see LICENSE.txt or git.io/sfdc-license

const _ = require('lodash');
const path = require('path');

module.exports = (basePath, directories = []) => {
  const patterns = ['']
    .concat(directories)
    .map(k => new RegExp(_.escapeRegExp(path.resolve(__dirname, k))));
  const shouldRemove = id =>
    patterns.reduce((allow, pattern) => allow || pattern.test(id), false);
  return id => {
    const m = require.cache[id];
    if (m) {
      delete require.cache[id];
      let parent = m.parent;
      while (parent && shouldRemove(parent.id)) {
        delete require.cache[parent.id];
        parent = parent.parent;
      }
    }
  };
};
